#!/bin/bash

############################################################################
#
# MODULE:       GUI
# AUTHOR(S):    CacheGuard Development Team
# COPYRIGHT:    (C) 2009-2020 by CacheGuard Technologies Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
###########################################################################

source functions

print-tls-client-component()
{
    test -n "${1}" || return 1
    test -n "${2}" || return 2
    local tls=${1}
    local component=${2}

    local file head title

    echo "<table class='report' width='100%'><tr><td>"
    echo "<pre>"

    case ${component} in
	certificate)
	    title=${component^}
	    file=${tls}.cur/${tls}.cert
	    ;;
	key)
	    title="Private ${component^}"
	    file=${tls}.cur/${tls}.key
	    ;;
	pkcs12)
	    title="Private ${component^^} bundle"
	    file=${tls}.cur/${tls}.p12
	    ;;
	pfx)
	    title="${component^^} (base 64 encoded form of the PKCS12)"
	    file=${tls}.cur/${tls}.pfx
	    ;;
	password)
	    title="PKCS12 ${component^}"
	    file=${tls}.cur/${tls}.passwd
	    ;;
	*)
	    ;;
    esac

    if test ! -f ${file} ; then
	echo-unavailable-message "This ${title} is not available."
    else
	case ${component} in
	    certificate)
		head=$(get-first-line ${file})
		if test "${head}" != "-----BEGIN CERTIFICATE-----" ; then
		    echo-unavailable-message "Warning: the loaded file does not seem to be a valid certificate file in PEM format."
		else
		    openssl x509 -in ${file} -text -noout 2> /dev/null
		    echo
		    openssl x509 -in ${file} -fingerprint -sha256 -noout 2> /dev/null
		    openssl x509 -in ${file} -fingerprint -sha1 -noout 2> /dev/null
		    echo
		    cat ${file}
		fi
		;;
	    key)
		head=$(get-first-line ${file})
		if test \
		       "${head}" != "-----BEGIN RSA PRIVATE KEY-----" -a \
		       "${head}" != "-----BEGIN PRIVATE KEY-----" ; then
		    echo-unavailable-message "Warning: the loaded file does not seem to be a valid private key file in PEM format."
		else
		    local sha1=$(openssl dgst -sha1 -c ${file} 2> /dev/null)
		    sha1=${sha1/*= }
		    sha1=${sha1^^}
		    echo "A private key can't be shown. The SHA1 fingerprint of this private key is:"
		    echo "${sha1}"
		fi
		;;
	    pkcs12)
		echo-unavailable-message "This ${title} is not in plain text format and thus can't be displayed here."
		;;
	    pfx)
		cat ${file} 2> /dev/null
		;;
	    password)
		echo "For security reasons the password protecting the PKCS12 and PFX can't be displayed here."
		;;
	    *)
		;;
	esac
    fi

    echo '</pre>'
    echo '</td></tr></table>'
}

print-tls-client()
{
    test -n "${1}" || return 1
    test -n "${2}" || return 2
    local tls=${1}
    local component=${2}

    cd ${CLIENT_SSL_DIR}

    case ${component} in
	key|certificate|pkcs12|pfx|password)
	    print-tls-client-component ${tls} ${component}
	    ;;
	*)
	    return 1
	    ;;
    esac
}

check-cgi-security
verify-authentication
test ${?} -eq 0 || http-exit 1
echo-http-header
print-tls-client "${@}"
