#!/bin/bash

############################################################################
#
# MODULE:       GUI
# AUTHOR(S):    CacheGuard Development Team
# COPYRIGHT:    (C) 2009-2020 by CacheGuard Technologies Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
###########################################################################

source functions

print-tls-component()
{
    test -n "${1}" || return 1
    test -n "${2}" || return 2
    local tls=${1}
    local component=${2}
    local ca=${3}

    local file head title

    echo "<table class='report' width='100%'><tr><td>"
    echo "<pre>"

    case ${component} in
	csr)
	    title=${component^^}
	    file=$(get-tls-component-file ${tls} csr)
	    ;;
	certificate)
	    title=${component^}
	    file=$(get-tls-component-file ${tls} cert ${ca})
	    ;;
	chain)
	    title="CA ${component^}"
	    file=$(get-tls-component-file ${tls} chain)
	    ;;
	key)
	    title="Private ${component^}"
	    file=$(get-tls-component-file ${tls} key ${ca})
	    ;;
	*)
	    ;;
    esac

    if test ! -f ${file} ; then
	echo-unavailable-message "This ${title} is not available."
    else
	if ! check-ascii-file ${file} ; then
	    echo-unavailable-message "The loaded file is not a plain text file and can't be displayed."
	else
	    case ${component} in
		csr)
		    head=$(get-first-line ${file})
		    if test "${head}" != "-----BEGIN CERTIFICATE REQUEST-----" ; then
			echo-unavailable-message "Warning: the loaded file does not seem to be a valid CSR file in PEM format."
		    else
			openssl req -in ${file} -text -noout 2> /dev/null
			echo
			cat ${file}
		    fi
		    ;;
		certificate)
		    head=$(get-first-line ${file})
		    if test "${head}" != "-----BEGIN CERTIFICATE-----" ; then
			echo-unavailable-message "Warning: the loaded file does not seem to be a valid certificate file in PEM format."
		    else
			openssl x509 -in ${file} -text -noout 2> /dev/null
			echo
			openssl x509 -in ${file} -fingerprint -sha256 -noout 2> /dev/null
			openssl x509 -in ${file} -fingerprint -sha1 -noout 2> /dev/null
			echo
			cat ${file}
		    fi
		    ;;
		chain)
		    head=$(get-first-line ${file})
		    if test "${head}" != "-----BEGIN CERTIFICATE-----" ; then
			echo-unavailable-message "Warning: the loaded file does not seem to be a valid chain certificate file in PEM format."
		    else
			local line begin end tmp=/tmp/tls-certificate.${$}
			rm -f ${tmp}
			while read line
			do
			    echo ${line} >> ${tmp}
			    if test "${line}" == "-----END CERTIFICATE-----" ; then
				openssl x509 -in ${tmp} -text -noout 2> /dev/null
				rm -f ${tmp}
			    fi
			done < ${file}
			echo
			cat ${file}
		    fi
		    ;;
		key)
		    head=$(get-first-line ${file})
		    if test \
			   "${head}" != "-----BEGIN RSA PRIVATE KEY-----" -a \
			   "${head}" != "-----BEGIN PRIVATE KEY-----" ; then
			echo-unavailable-message "Warning: the loaded file does not seem to be a valid private key file in PEM format."
		    else
			local sha1=$(openssl dgst -sha1 -c ${file} 2> /dev/null)
			sha1=${sha1/*= }
			sha1=${sha1^^}
			echo "A private key can't be shown. The SHA1 fingerprint of this private key is:"
			echo "${sha1}"
		    fi
		    ;;
		*)
		    ;;
	    esac
	fi
    fi

    echo '</pre>'
    echo '</td></tr></table>'
}

print-tls-conf()
{
    test -n "${1}" || return 1
    local tls=${1}
    local ca=${2}

    local conf_file var val
    local days names country province locality organisation unit

    conf_file=$(get-tls-component-file ${tls} conf ${ca})

    if test ! -f ${conf_file} ; then
	echo -n "${names}|${days}|${country}|${province}|${locality}|${organisation}|${unit}"
	return 3
    fi

    while read var val
    do
	case ${var} in
	    commonName)
		names=${val}
		;;
	    days)
		days=${val}
		;;
	    countryName)
		country=${val}
		;;
	    stateOrProvinceName)
		province=${val}
		;;
	    localityName)
		locality=${val}
		;;
	    organizationName)
		organisation=${val}
		;;
	    organizationalUnitName)
		unit=${val}
		;;
	    *)
		;;
	esac
    done < ${conf_file}

    echo -n "${names}|${days}|${country}|${province}|${locality}|${organisation}|${unit}"
}

print-tls()
{
    test -n "${1}" || return 1
    test -n "${2}" || return 2
    local tls=${1}
    local component=${2}
    local ca=${3}

    case ${component} in
	conf)
	    print-tls-conf ${tls} ${ca}
	    ;;
	key|csr|certificate|chain)
	    print-tls-component ${tls} ${component} ${ca}
	    ;;
	*)
	    return 1
	    ;;
    esac
}

check-cgi-security
verify-authentication
test ${?} -eq 0 || http-exit 1
echo-http-header
print-tls "${@}"
